; Exposure time Calculator for the MOSFIRE instrument on Keck I
; Written by Gwen C. Rudie (gwen@astro.caltech.edu)

; band: name of band
; slit_width: width of slit in arcseconds
; Nreads: number of reads in fowler
; theta: angular size of the object along the slit
; nExp: the number of exposures: if nEXP > 2, assumes a 2 point dither
; magnitude
; AB - set if the magnitude is in AB
; lineF: line flux in 10^-18 erg/s/cm^2
; lineW: line wavelength of interest (rest or observed)
; Angstroms: set to 1 if the line width is in Angstroms
; FWHM: rest-frame line FWHM in km/s
; z: redshift of line of interest
; theta: angular size of the object along the slit
; time: exposure time in seconds
; SN: desired signal to noise
; GUI: the GUI state so we can print to the GUI
; Airmass: the airmass value you want to use for the sky bkgrd
; wvcol: the water vapor column to use for the sky bkgrd
; output_struct, a structure storing the output spectra for plotting
;                and printing
;specFile, if set, this is the filename for the micron, F_nu spectrum 
;          that the user input. Integrate the value, make it match the
;          magnitude they input, and then use for the SN calculation

;UPDATES: (1) Keck Mirror relectance updated 6/26/2012 by GCR
;         (2) Throughput changed to the measured throughput.
;             Measurements by CCS, code by GCR 6/26/2012
;         (3) Sky Background Spectra updated to sky spectra taken with
;             MOSFIRE. Spectra from CCS, code by GCR 6/26/2012
;             ATM transmission spectra still from Gemini
;         (4) Read Noise takes into account the number of exposures,
;             code by GCR 6/27/2012
;         (5) The values reported for the continuum S/N calculation
;             were changed from averages across the band to medians across
;             the band.
;         (6) Currently the S/N etc are calculated over the portion of
;             sky spectrum that is not zero-valued as opposed to being
;             over the full band pass of the filter. 
;         (7) Changed K band spectrum out for the longer wavelength coverage



;NEEDED UPDATES: (1) Current Sky Spectrum is cropped at long
;                    wavelengths in H band and on both edges of the K
;                    band

; XTcalc, 'K', 0.7, 16, lineF=9.0, lineW=6563, /Angstroms, FWHM=90, theta=0.7, z=2.3, time=1000
; XTcalc, 'K', 0.7, 16, mag=18.6, time=1000


pro XTcalc, band, slit_width, Nreads, theta, nExp, mag=mag, AB=AB, $
            lineF=lineF, lineW=lineW, Angstroms=Angstroms,  $
            FWHM=FWHM, z=z, time=time, SN=SN, GUI=GUI, airmass=airmass, wvcol=wvcol, $
            output_struct=output_struct, specFile=specfile, specStruc=specStruc, ok=ok


  ;if the number of exposures is greater than 1, assume two dither positions
  if (nExp gt 1) then dither=2.0 else dither=1.0

  ;the location of the code and the data spectra
  usr_path=getenv('MOSFIRE_XTCALC')
  
  mos_path=usr_path+'/mosfire/'
  sky_path=usr_path+'/Mauna_Kea_sky/'
  tp_path=usr_path+'/MosfireSpecEff/'
  bk_path=usr_path+'/MosfireSkySpec/'

  ;put everything in micron
  if (keyword_set(lineF) and keyword_set(Angstroms)) then lineW=lineW/10000.0

  ok=1

  ;***************************************************
  ;***************************************************
  ;       MOSIFRE Linearity Limits
  ;       2.15 e/ADU gain
  ;       1% non linearity 26K ADU= 55900 electrons
  ;       5% non linearity 37K ADUs = 79550 e-
  ;       saturation 43K ADUs = 92450 e-
  ;***************************************************
  ;***************************************************

  one_per_limit=55900
  five_per_limit=79550
  sat_limit=92450

  ;*************************************
  ;*************************************
  ;       MOSIFRE Filters
  ;*************************************
  ;*************************************

  ;read in the filter curve - wavlegth in microns
  readcol, mos_path+'mosfire_'+band+'.txt', filt_wave, filt, format='f,f', /silent


  ;check to see if the band is correct
  if keyword_set(lineW) then begin
     line=lineW*(1+z)

     if ((line gt max(filt_wave)) or (line lt min(filt_wave))) then begin
        ok=0

        if keyword_set(GUI) then begin
           longString=String(Format=$
                             '("This wavelength (",(f0.4)," micron) is not contained in ",(A)," band.")',line, band) 
           widget_control, GUI.output_text, set_value=LongString
           display_warning, LongString

        endif else print, String(Format='("This wavelength (",(f0.4)," micron) is not contained in ",(A)," band.")',line, band)
        
     endif
  endif


  ;*************************************
  ;*************************************
  ; check the user input spectrum
  ;*************************************
  ;*************************************

  if keyword_set(specFile)  then begin

     if (keyword_set(specStruc) eq 0) then begin

        readcol, specFile, userWave, userFlux, format='f,f', delimiter=' ,:'

     endif else begin

        userWave=specStruc.wave
        userFlux=specStruc.flux

     endelse

     if keyword_set(Angstroms) then userWave=userWave/10000.0

     userWave=userWave*(1+z)

     real_filt_wave=filt_wave(where(filt gt 0.01))

     ;does the user spectrum cover the full band pass
     ;and are the wavelengths in micron
     if ((min(userWave) gt min(real_filt_wave)) or (max(userWave) lt max(real_filt_wave))) then begin
        
        ok=0
        
        warning=['The read-in spectrum from '+specFile,$
                 'does not span the full wavelength coverage of the '+band+' band',$
                 'or is not in the proper format. The correct format is ',$
                 'observed-frame wavelength in micron or Angstroms and flux in',$
                 'erg/s/cm'+String(178B)+' in two column format with a space or comma',$
                 'as the delimiter. Also please check that you have choosen the corect',$
                 '"wavelength unit" on the GUI.']

        display_warning, warning

     endif

  endif

  if (ok ne 0) then begin

  ;if the magnitude is entered in Vega, change it to AB.
  ; conversions for J,H,Ks from Blanton et al 2005
  ; K from ccs
  ; Y from CFHT WIRCam
  if (keyword_set(mag) and (keyword_set(AB) eq 0)) then begin
     if (band eq 'Y') then mag=mag+0.66 $
     else if (band eq 'J') then mag=mag+0.91 $
     else if (band eq 'H') then mag=mag+1.39 $
     else if (band eq 'K') then mag=mag+1.95 $              
     else if (band eq 'Ks') then mag=mag+1.85                     

  endif 


  ;*************************************
  ;*************************************
  ;         CONSTANTS
  ;*************************************
  ;*************************************

  ;the speed of light in cm/s
  c=alog10(29979245800.D)

  ;planks constant in erg*s
  h=alog10(6.626068)-27

  ;in log(erg cm)
  loghc=c+h

  f_nu_AB=48.59


  ;*************************************
  ;*************************************
  ;     KECK and MOSFIRE CONSTANTS
  ;*************************************
  ;*************************************

  ;area of the telescope in cm^2
  AT=750000.0

  ;slit width used in arcsec to measure R_theta
  slit_W=0.7

  ;spatial pixel scale in arcseconds/pixel
  pix_scale=0.18

  ;dispersion pixel size
  pix_disp=0.24

  ;Number of pixels in dispersion direction on the detector
  tot_Npix=2048.

  ;Detector readnoise in electrons/pix CDS (correlated double sampling)
  det_RN=15.

  ;number of elements
  Nelement=13.

  ;number of MOSFIRE mirrors
  Nmirror=1.

  ;number of windows
  Nwindow=4.

  ;*************************************
  ;*************************************
  ;         SKY CONSTANTS
  ;*************************************
  ;*************************************

  if (keyword_set(airmass) and keyword_set(wvcol)) then begin
     ;sky_spec='mk_skybg_zm_'+wvcol+'_'+airmass+'_ph.dat' 
     ;transpec='mktrans_zm_'+wvcol+'_'+airmass+'.dat'
     ;sky_spec='mk_skybg_zm_'+wvcol+'_'+airmass+'_ph.sav' 
     transpec='mktrans_zm_'+wvcol+'_'+airmass+'.sav'
  endif else begin 
     ;sky_spec='mk_skybg_zm_16_10_ph.dat'
     ;transpec='mktrans_zm_16_10.dat'
     ;sky_spec='mk_skybg_zm_16_10_ph.sav'
     transpec='mktrans_zm_16_10.sav'
  endelse

  sky_spec=band+'sky_cal_pA.sav'

;  UPDATED on 6/26/2012
;  ;read in the night sky from Mauna Kea:
;  ;From Gemini Observatory: Lord (1992)
;  ;default is 1.6mm water vapor column
;  ;armiass=1
;  ;units are: photons/sec/arcsec^2/nm/m^2
;  restore, sky_path+sky_spec
;  ;readcol, sky_path+sky_spec, lam, mkSky, format='f,f', /silent
;  lam=lam/1000.0    


  ;read in the night sky from Mauna Kea as measured with MOSFIRE
  ;Measured sky spectrum from CCS
  ;with units of ergs/s/cm^2/A per spatial pixel for a 0.7" slit
  ;mksky=rflam(bk_path+sky_spec,hdr,lam)
  ;save, lam, mksky, filename=bk_path+band+'sky_cal_pA.sav'
  
  ;make the Kband spectrum
  ;f=rflam('Ksky_cal_pp.wide.fits', hdr, lam)
  ;mksky=f/Kstat.disp
  ;save, lam, mksky, filename=bk_path+band+'sky_cal_pA.sav'

  restore, bk_path+sky_spec
  lam=lam/10000.
  ;convert to the same units as the old gemini spectrum
  ;photons/sec/arcsecond^2/nm/m^2
  ;divide by slit width and angular size of pixel to get per arcsec^2
  ;divide by hc/lam where hc is in (erg micron)
  ;multiply by 10^4 for per cm^2 to m^2 and 10 for per nm to A
  mksky=mksky/(0.7*pix_scale*10.^(loghc+4.0))*lam*10^(4.+1.)

  zero_index=where(mksky lt 0,ct_index)
  if (ct_index gt 0) then mksky[zero_index]=0

  ;read in the atmospheric transparency
  ;From Gemini Observatory: Lord, S. D., 1992, NASA Technical Memorandum 103957
  ;default is 1.6mm water vapor column
  ; armiass=1
  restore, sky_path+transpec
  ;readcol, sky_path+transpec, tran_lam, trans, format='f,f', /silent


  ;*************************************
  ;*************************************
  ;       MOSIFRE Throughput
  ;*************************************
  ;*************************************

  ;elem_AR = elment AR per surface
  ;ref_mir = mirror reflectance
  ;win_AR = window AR per surface
  ;grat_eff: grating effciency
  ;fil_tran: filter transmission
  ;qe: quantum efficiency
  ;disp: dispersion in angstroms/pixel
  ;cent: central wavelength in micron
  ;background in magnitudes per arcsecond^2
  ;f_not: log of magnitude zero point in erg/s/cm^2/micron
  ;dark current in electrons per second
  ;rt: R-theta product: multiply by slit width to get resolution
  ;KMRef: Keck Mirror reflectance


  Kstat={elem_AR:0.99, $
         ref_mir:0.98, $
         win_AR:0.988, $
         grat_eff:0.72,$
         fil_trans:0.93, $
         qe:0.88, $
         disp:2.10, $
         lambda:2.2, $
         background:16, $
         f_not:alog10(3.8)-7, $
         dark: 0.005, $
         rt:3620.0*slit_W, $
         KMRef:.95 $
          }

  Hstat={elem_AR:0.992, $
         ref_mir:0.985, $
         win_AR:0.985, $
         grat_eff:0.65,$
         fil_trans:0.95, $
         qe:0.88, $
         disp:1.73, $
         lambda:1.65, $
         background:16.6, $
         f_not:alog10(1.08)-6, $
         dark: 0.005, $
         rt:3660.0*slit_W, $
         KMRef:.94 $
          }

  Jstat={elem_AR:0.985, $
         ref_mir:0.98, $
         win_AR:0.985, $
         grat_eff:0.8,$
         fil_trans:0.9, $
         qe:0.8, $
         disp:1.31, $
         lambda:1.25, $
         background:16.8, $
         f_not:alog10(2.90)-6, $
         dark: 0.005, $
         rt:3310.0*slit_W, $
         KMRef:.89 $
          }

  Ystat={elem_AR:0.985, $
         ref_mir:0.98, $
         win_AR:0.985, $
         grat_eff:0.8,$
         fil_trans:0.9, $
         qe:0.8, $
         disp:1.05, $
         lambda:1.05, $
         background:17.3, $
         f_not:alog10(7.45)-6, $
         dark: 0.005, $
         rt:3380.0*slit_W, $
         KMRef:0.85 $
          }

  ;*************************************
  ;*************************************
  ;*************************************
  ;*************************************


 
  if (band eq 'K') then stat=Kstat else $
     if (band eq 'H') then stat=Hstat else $
        if (band eq 'J') then stat=Jstat else $
           if (band eq 'Y') then stat=Ystat 
  

  ;**************************
  ;figure out the throughput
  ;**************************

  ;read in the throughput spectrum UPDATED 6/26/2012
;  readcol, mos_path+band+'_tp_tot.txt', tp_wave, throughput, format='f,f', /silent
  readcol, tp_path+band+'eff.sm.dat', tp_wave, throughput, format='f,f', /silent
  tp_wave=tp_wave/10000.

  zero_index=where(throughput lt 0, zero_count)

  if (zero_count gt 0) then throughput[zero_index]=0.0

  ;number of Keck Mirrors (Primary and Secondary)
  NKmir = 2

  ;the total throughput is the instrument throughput
  ;times the reflectance of the keck primary and secondary
  throughput=throughput*stat.KMRef^(NKmir)


  ;real FWHM resolution
  R=stat.rt/slit_width
  
  ;slit width in pixels along the dispersion direction
  swp=slit_width/pix_disp

  ;spectral coverage in micron)
  cov=tot_Npix*(stat.disp/10000.)

 
  ;*************************************
  ;*************************************
  ;     Using a specific line flux
  ;*************************************
  ;*************************************
  if keyword_set(lineF) then begin

     ;figure out the relavant wavelength range

     ;the observed line wavelength in micron
     center=lineW*(1+z)

     ;resolution at the central wavelength in micron
     res=center/R

     ;the width of the spectral line before going through the spectrograph
     real_width=center*FWHM*10^(-c+5)
    
     ;the line width in micron that would be observed
     width=sqrt(real_width^2+res^2)


  endif else begin

     ;*************************************
     ;*************************************
     ; we are calculating for a broad band flux
     ;*************************************
     ;*************************************

     center=stat.lambda

     ;resolution at the central wavelength in micron
     res=center/R

  endelse 

  ;convolve the filter with mosfire resolution
  mosfire_resolution, filt_wave, filt, stat.lambda, R, stat.disp, out_wave=wave_grid, out_flux=fltSpecObs

  ;the relavant portion of the spectrum
  band_index=where(fltSpecObs gt 0.1)

  fltSpecObs=fltSpecObs[band_index]

  ; a tighter contraint for the S/N
  filt_index=where(fltSpecObs gt 0.5)
  
  ;convolve the atm_Transmission spectrum with the resolution
  mosfire_resolution, tran_lam, trans, stat.lambda, R, stat.disp, out_wave=wave_grid, out_flux=tranSpecObs

  tranSpecObs=tranSpecObs[band_index]

  ;convolve the throughput spectrum with the resolution
  mosfire_resolution, tp_wave, throughput, stat.lambda, R, stat.disp, out_wave=wave_grid, out_flux=tpSpecObs

  tpSpecObs=tpSpecObs[band_index]

  ;convolve the background spectrum with the resolution
  ;background in phot/sec/arcsec^2/nm/m^2
  mosfire_resolution, lam, mksky, stat.lambda, R, stat.disp, out_wave=wave_grid, out_flux=raw_bkSpecObs
  zero_index=where(raw_bkSpecObs lt 0,ct_index)
  NONzero_index=where((raw_bkSpecObs ge 0) and (raw_bkSpecObs ne -0))
  if (ct_index gt 0) then raw_bkSpecObs[zero_index]=0

  raw_bkSpecObs=raw_bkSpecObs[band_index]
  wave_grid=wave_grid[band_index]

  ;for now - sent the filt_index to be the NONzero_index
  filt_index=NONzero_index

  ;*************************************
  ;*************************************
  ;     Using a specific line flux
  ;*************************************
  ;*************************************
  if keyword_set(lineF) then begin

     ;the location inside the FWHM of the line
     line_index=where(abs(wave_grid - center) le 0.5*width)

     ; the area used to caluclate the S/N
     sn_index=line_index

     ;now send the background spectrum through the telescope by 
     ;multiplying the throughput, the
     ;slit_width, the angular extent (theta), the area of the
     ;telescope, and the pixel scale in nm

     ;this gives phot/sec/pixel
     bkSpecObs = raw_bkSpecObs * tpSpecObs * slit_width *theta * (AT*10.^(-4)) * (stat.disp/10.0)
;     bkSpecObs = raw_bkSpecObs * tpSpecObs * (0.5)^2* (AT*10.^(-4)) * (stat.disp/10.0)*10^(0.4*1.3)

     ;determine the average background
     ;within the FWHM of the line 

     ;this is no longer true
     ;in photons per second per pixel per Ang.

     ;in photons per second per arcsec^2 per nm per m^2
     mkBkgd=mean(mkSky[where(abs(lam-center) le 0.5*width)])
     
     ;fv_back=mkBkgd*(center)^2*10^(-4+3-4-c)

     ;what does this correspond to in AB mags/arcsec^2

     ;go to erg/s/cm^2/Hz
     ;10^-4 for m^2 to cm^2
     ;10^3 for micron to nm
     ;lam^2/c to covert from d(lam) to d(nu) (per Hz instead of per nm)
     ;hc/lam to convert photons to ergs
     mag_back=-2.5*(alog10(mkBkgd*center)-4+3+h)-f_nu_AB
   
  

     ; the signal in electrons per second that will hit the telsecope
     ; as it hits the atmosphere (ie need
     ; to multiply by the throughput and
     ; the atmospheric transparency
     signalATM = lineF * 10^(-18-loghc-4) * center * AT
     
                     ;don't need the FWHM part
     ;signal = lineF * 10^(-18-loghc-4) * AT * center *erf(sqrt(alog(2)))
     ; wave_grid_res=mean(wave_grid[1:n_elements(wave_grid)-1]-wave_grid[0:n_elements(wave_grid)-2])


     ;the width of the line in sigma - not FWHM
     ;in micron
     sigma=width/(2*sqrt(2*alog(2)))

     ;a spectrum version of the signal
     ;phot per second per pixel (without atm or telescope)
     ; ie total(signal_spec/signal) with
     ; equal resolution of wave_grid /
     ; stat.disp in micron
     signal_spec=signalATM*(1/(sqrt(2*!pi)*sigma))*exp(-0.5*(wave_grid-center)^2/sigma^2)*stat.disp/10.^4


     ; the spectrum of the signal as detected
     sigSpecObs=signal_spec * tpSpecObs * tranSpecObs




     ; the nubber of pixels in the spectral direction
     nPixSpec = (width*10000.0)/stat.disp

     ;the spatial pixel scale
     nPixSpatial=theta/pix_scale

     ;The number of pixels per FWHM observed
     Npix= nPixSpec * NpixSpatial

  endif else begin
     ;*************************************
     ;*************************************
     ; we are calculating for a broad band flux
     ;*************************************
     ;*************************************


     ; the area used to caluclate the S/N
     sn_index=filt_index


     mag_back=-2.5*(alog10(mean(raw_bkSpecObs[filt_index])*center)-4+3+h)-f_nu_AB


     ;now send the background spectrum through the telescope by 
     ;multiplying the throughput, the
     ;slit_width, the angular extent, the area of the
     ;telescope, and the pixel scale in nm

     ;this gives phot/sec/pixel
     bkSpecObs = raw_bkSpecObs * tpSpecObs * slit_width * theta * (AT*10.^(-4)) * (stat.disp/10.0)
;     bkSpecObs = raw_bkSpecObs * tpSpecObs * (0.5)^2 * (AT*10.^(-4)) * (stat.disp/10.0)



     ;*************************************
     ;*************************************
     ; using the user input spectrum
     ;*************************************
     ;*************************************

     if keyword_set(specFile) then begin

        ;convolve with the resolution of mosfire
        mosfire_resolution,  userWave, userFlux, stat.lambda, R, stat.disp, $
                             out_wave=user_wave_grid, out_flux=userSig
        userSig=userSig[band_index]

        ;multiply by the normalized filter
        ;transmission 

        filt_shape=fltSpecObs/max(fltSpecObs)
        userSig=userSig*filt_shape

        ;make the total match
        ;the broad band magnitude
        scale=10.0^(-0.4*(mag[0]+f_nu_AB))/mean(userSig)

        ;raw fv spec
        raw_fv_sig_spec=userSig*scale


        ;convert to flux hitting the primary
        ;in flux hitting the primary in
        ;phot/sec/micron 
        ;(if the earth had no atmosphere)
        ;phot/sec/micron = fnu * AT / lam / h
        signal_spec=raw_fv_sig_spec*10.^(-1*h)*AT / wave_grid


     endif else begin
     ;*************************************
     ;*************************************
     ; using a flat F_nu spec (DEFAULT)
     ;*************************************
     ;*************************************
        
         ;fv=10^((-2/5)*MagAB-48.59) (erg/s/cm^2/Hz)
         ;convert to flam: flam=fv*c/lam^2 (erg/s/cm^2/micron)
         ;covert to photons: phot/sec/micron = fnu * AT / lam / h

        ;flux hitting the primary in
        ;phot/sec/micron (if the earth had no atmosphere)
        signal_spec=10.0^(-0.4*(mag[0]+f_nu_AB)-h) * AT / wave_grid

       ; newSpec=10.0^(-0.4*(mag[0]+f_nu_AB)) * wave_grid/wave_grid

     endelse


     ;multiply by the atmospheric transparency
     signal_spec=signal_spec * tranSpecObs

     ; now put it through the throughput of the telescope
     ; phot/sec/micron
     sigSpecObs= signal_spec * tpSpecObs

     ; now for phot/sec/pix multiply by micron/pix
     sigSpecObs=sigSpecObs * (stat.disp/10000.0)

     ;number of pixels per resolution element in the 
     ;spectral direction

     nPixSpec = (res*10000.0)/stat.disp

     ;the spatial pixel scale
     nPixSpatial=theta/pix_scale

     ;The number of pixels per FWHM observed
     Npix= nPixSpec * NpixSpatial


    

     ;*************************************
     ;*************************************
     ; NEED TO ADD SLIT LOSSES FOR OBJECT
     ;*************************************
     ;*************************************


  endelse


  if keyword_set(SN) then begin


     ; differentiate between total exposure time 
     ; and amount of time of individual exposures
    
;     ; figure out how long it takes
;     qb=bkSpecObs + stat.dark*nPixSpatial + sigSpecObs
;     qa=-nPixSpec * sigSpecObs^2/SN[0]^2
;     qc=det_RN^2/Nreads*nPixSpatial

     ; figure out how long it takes

     ;if calulating with a line flux, assume S/N over the line
     ; other wise, S/N per spectral pixel
     if keyword_set(lineF) then qa=-nPixSpec * sigSpecObs^2/SN[0]^2 $
     else qa=-sigSpecObs^2/SN[0]^2

     qb=dither*bkSpecObs + dither*stat.dark*nPixSpatial + sigSpecObs
     qc=dither*det_RN^2/Nreads*nPixSpatial*nExp

     timeSpec=(-qb - sqrt( qb^2 - 4*qa*qc ))/(2*qa)
     
     time=median(timeSpec[sn_index]) 

     time=float(time)

  endif

  ; the signal to noise

  ; noise contributions
  ; poisson of background
  ; poisson of dark current
  ; poisson of the signal
  ; read noise


  ;the noise per slit length in the spatial direction
  ; and per pixel in the spectral direction

  ; the noise spectrum: 
  ; Poisson of the dark
  ; current, signal, and background + the read noise"

  noiseSpecObs = sqrt( sigSpecObs*time[0] + dither*( (bkSpecObs +stat.dark*nPixSpatial)*time[0] +$
                                                     det_RN^2/Nreads*nPixSpatial*nExp))


;  noiseSpecObs = sqrt((bkSpecObs +stat.dark*nPixSpatial + sigSpecObs)*time[0] +det_RN^2/Nreads*nPixSpatial)

  signalSpecObs =  sigSpecObs * time[0]

  snSpecObs = signalSpecObs / noiseSpecObs

  stn = mean(sqrt(nPixSpec) * snSpecObs[sn_index])

  ;the electron per pixel spectrum
  eppSpec=noiseSpecObs^2/nPixSpatial

  ;*************************************
  ;*************************************
  ;       values to be printed
  ;*************************************
  ;*************************************
  
  ;the mean instrument+telescope throughput in
  ;the same band pass
  tp = mean(tpSpecObs[sn_index])

  ;maximum electron per pixel
  max_epp=max(eppSpec[sn_index]/nExp)


  ;if calulating a line flux, S/N per FWHM 
  ;ie S/N in the line


  if keyword_set(lineF) then begin

     ;over the line (per FWHM)
     stn = mean(sqrt(nPixSpec) * snSpecObs[sn_index])

     ;signal in e/FWHM
     signal = mean(sigSpecObs[sn_index])*nPixSpec*time[0]

     ;sky background in e/sec/FWHM
     background = mean(bkSpecObs[sn_index])*nPixSpec*time[0]
 
     ;Read noise for multiple reads, electrons per FWHM
     RN=det_RN/sqrt(Nreads)*sqrt(Npix)*sqrt(nExp)

     ;noise per FWHM
     noise=mean(noiseSpecObs[sn_index])*sqrt(nPixSpec)

     ;e- 
     dark=stat.dark*Npix*time[0]


  endif else begin
     ;we are computing S/N per pixel for a continuum source

     ;per spectral pixel
     stn = median(snSpecObs[sn_index])

     ;signal in e/(spectral pixel)
     signal = median(sigSpecObs[sn_index])*time[0]

     ;sky background in e/(spectral pixel)
     background = median(bkSpecObs[sn_index])*time[0]
 
     ;Read noise for multiple reads, electrons per spectral pixel
     RN=det_RN/sqrt(Nreads)*sqrt(nPixSpatial)*sqrt(nExp)

     ;noise per spectral pixel
     noise=median(noiseSpecObs[sn_index])

     ;e- per spectral pixel
     dark=stat.dark*nPixSpatial*time[0]


  endelse

     


  ;*************************************
  ;*************************************
  ;      display the results
  ;*************************************
  ;*************************************



  if keyword_set(GUI) then begin

     struct={quant:'',value:'', unit:''}
  
     out_struct=replicate(struct,13)

     out_struct.quant=['Wavelength', 'Resolution','Dispersion', 'Throughput', 'Signal', 'Sky Background', $
                       'Sky brightness', 'Dark Current', 'Read Noise', 'Total Noise','S/N', $
                       'Exposure Time', 'Max e- per pixel']

     if keyword_set(lineF) then out_struct.unit=$
        ['micron','FWHM in angstrom', 'angstrom/pixel', '',  'electrons per FWHM',$
         'electrons per FWHM', 'AB mag per sq. arcsec', 'electrons per FWHM', $
         'electrons per FWHM', $
         'electrons per FWHM',$
         'per observed FWHM', 'seconds', 'electrons per pixel per exp'] $
     else out_struct.unit=$
        ['micron','angstrom', 'angstrom/pixel', '',  'electrons per spectral pixel',$
         'electrons per spectral pixel', 'AB mag per sq. arcsec', 'electrons per spectral pixel', $
         'electrons per spectral pixel', 'electrons per spectral pixel',$
         'per spectral pixel', 'seconds', 'electrons per pixel'] 

     if (max_epp ge 10.^10) then max_epp_string='>'+String(Format='(I11)',10.^10) $
     else max_epp_string=String(Format='(I10)',max_epp)

     out_struct.value=[String(Format='(f0.4)',center),String(Format='(f0.1)',res*10^4),$
                       String(Format='(f0.2)',stat.disp), String(Format='(f0.2)',tp), $
                       String(Format='(f0.2)',signal),String(Format='(f0.2)',background),$
                       String(Format='(f0.2)',mag_back),String(Format='(f0.2)',dark), $
                       String(Format='(f0.2)',RN),String(Format='(f0.2)',noise),$
                       String(Format='(f0.1)',stn),String(Format='(f0.2)',time[0]), $
                       max_epp_string]

     

     widget_control, GUI.table, set_value=out_struct
    
     ;turn the max e- per pixel red if the detector has saturated
     color_bkgrd=fltarr(3,13*3)+255
     if (max_epp gt sat_limit) then color_bkgrd[1:2,36:38]=0 else $
     if (max_epp gt five_per_limit) then begin
        color_bkgrd[2,36:38]=0 
        color_bkgrd[1,36:38]=140
        endif else $
     if (max_epp gt one_per_limit) then color_bkgrd[2,36:38]=0

     widget_control, GUI.table, BACKGROUND_COLOR=color_bkgrd

     if keyword_set(SN) then begin
        Longstring=["*****************************************",$
                    "             MOSFIRE XTCalc              ",$
                    "*****************************************",$
                    String(Format='("Calculation for a signal to noise ratio of ",(f0.1))', stn),$
                    String(Format='("through a ",(f0.1)," arcsecond slit in ",(A)," band")',slit_width,band)]
     endif else begin
        Longstring=["*****************************************",$
                    "             MOSFIRE XTCalc              ",$
                    "*****************************************",$
                    String(Format='("Calculation for a ",(f0.2)," second integration ")', time),$
                    String(Format='("through a ",(f0.1)," arcsecond slit in ",(A)," band")',slit_width,band)]
     endelse


     widget_control, GUI.output_text, set_value=LongString

    
 
     output_struct={wave_ptr:ptr_new(wave_grid), $
                    center:center, $
                    plot_index_ptr:ptr_new(sn_index),$
                    filt_index_ptr:ptr_new(filt_index), $
                    tp_ptr:ptr_new(tpSpecObs), $
                    filt_ptr:ptr_new(fltSpecObs), $
                    tran_ptr:ptr_new(tranSpecObs), $
                    bk_ptr:ptr_new(bkSpecObs), $
                    sig_ptr:ptr_new(sigSpecObs), $
                    signal_ptr:ptr_new(signalSpecObs), $
                    noise_ptr:ptr_new(noiseSpecObs), $
                    sn_ptr:ptr_new(snSpecObs), $
                    lineF:keyword_set(lineF), $
                    time:time[0]}
      
     ;and make a graph
    
     plot_obs, output_struct,  0, graph_window=GUI.graph 


;     plot_sn, output_struct, graph_window=GUI.graph





   endif else begin




     print, "*************************************"
     print, "*************************************"
     print, "         MOSFIRE XTCalc              "
     print, "*************************************"
     print, "*************************************"
     
     print, "Calculation for a "+strtrim(string(fix(time)),1)+" second integration "
     print, "through a "+strtrim(string(slit_width),1)+" arcsecond slit in "+band+" band"
     print, ""
     
     print, "Resolution: "+strtrim(string(res*10^4),1)+" (FWHM in angstrom)"
     print, "Sampling: "+strtrim(string(stat.disp),1)+" (angstrom/pixel)"
     print, "Throughput: "+strtrim(string(tp),1)
     print, ""
     
     
     print, "Values calculated at "+strtrim(string(center),1)+" micron"
     print, "Signal: "+strtrim(string(signal),1)+" (electrons per second)"
     print, "Sky Background: "+strtrim(string(background),1)+" (electrons per second)"
     print, "Sky brightness in magnitudes: "+strtrim(string(mag_back),1)+" (AB mag/sq. arcsecond)"
     print, "Dark Current: "+strtrim(string(stat.dark),1)+" (electrons per second)"
     print, "Read Noise: "+strtrim(string(RN),1)+" (electrons)"
     print, "Time: "+strtrim(string(time),1)+" (seconds)"
     print, " "
     print, "S/N: "+strtrim(string(stn),1)+" (per resolution element)"
     
  endelse

endif


end


pro mosfire_resolution, in_wave, in_flux, center_wave, R, disp, out_wave=out_wave, out_flux=out_flux

 ;Number of pixels to be output - 50%
 ; more than are on the detector to
 ; cover the K band
  Npix_spec=2048.*3./2.

  ;the speed of light in cm/s
  c=alog10(29979245800.D)

  ;make a "velocity" grid centered at
  ;the central wavelength of the band
  ; sampled at 1 km/s
  vel=(findgen(200001)-100000)

  in_vel=(in_wave/center_wave -1)*10.^(1*c-5)

  in_vel_short=in_vel[(where((in_vel gt vel[0]) and (in_vel lt vel[200000])))]
  in_flux_short=in_flux[(where((in_vel gt vel[0]) and (in_vel lt vel[200000])))]

  interp_flux=interpol(in_flux_short, in_vel_short, vel)

  ;sigma  = the resolution of the spectrograph
  sigma=(10.^(c-5)/R)/(2*sqrt(2*alog(2)))

  ;make a smaller velocity array with
  ;the same "resolution" as the steps in
  ;vel, above
  n = round(8*sigma)
  if n mod 2 eq 0 then n += 1
  vel_kernal=findgen(n)-floor(n/2.0)

  ;a gaussian of unit area and width sigma
  gauss_kernal=(1/(sqrt(2*!pi)*sigma))*exp(-0.5*vel_kernal^2/sigma^2)

  convol_flux=convol(interp_flux, gauss_kernal) 
  convol_wave = center_wave*(vel*10.^(-1*c+5) + 1 )


  ;and the real pixel scale of mosfire
  real_wave=findgen(Npix_spec)*disp*10.^(-4.)
  real_wave=real_wave-real_wave[round(Npix_spec/2.)]   
  real_wave=real_wave+center_wave 

  ;interpolate onto the pixel scale of the detector
  out_wave=real_wave
  out_flux=interpol(convol_flux, convol_wave, real_wave)


end

;pro mosfire_resolution_obsolete, in_wave, in_flux, center_wave, R, out_wave=out_wave, out_flux=out_flux
;
;  ;the speed of light in cm/s
;  c=alog10(29979245800.D)
;
;  ;make a "velocity" grid centered at
;  ;the central wavelength of the band
;  ; sampled at 1 km/s
;  vel=(findgen(200001)-100000)
;
;  in_vel=(in_wave/center_wave -1)*10.^(1*c-5)
;
;  in_vel_short=in_vel[(where((in_vel gt vel[0]) and (in_vel lt vel[200000])))]
;  in_flux_short=in_flux[(where((in_vel gt vel[0]) and (in_vel lt vel[200000])))]
;
;  interp_flux=interpol(in_flux_short, in_vel_short, vel)
;
;  ;sigma  = the resolution of the spectrographplot, 
;  sigma=(10.^(c-5)/R)/(2*sqrt(2*alog(2)))
;
;  ;make a smaller velocity array with
;  ;the same "resolution" as the steps in
;  ;vel, above
;  n = round(8*sigma)
;  if n mod 2 eq 0 then n += 1
;  vel_kernal=findgen(n)-floor(n/2.0)
;
;  ;a gaussian of unit area and width sigma
;  gauss_kernal=(1/(sqrt(2*!pi)*sigma))*exp(-0.5*vel_kernal^2/sigma^2)
;
;  out_flux=convol(interp_flux, gauss_kernal)
;  
;  out_wave = center_wave*(vel*10.^(-1*c+5) + 1 )
;
;end


pro plot_obs, struct, xrange, graph_window=graph_window

  ;make the structure into something useful
  wave_grid=*struct.wave_ptr
  sn_index=*struct.plot_index_ptr
  filt_index=*struct.filt_index_ptr
  tpSpecObs=*struct.tp_ptr
  fltSpecObs=*struct.filt_ptr
  tranSpecObs=*struct.tran_ptr
  bkSpecObs=*struct.bk_ptr
  sigSpecObs=*struct.sig_Ptr
  center=struct.center
  time=struct.time
  lineF=struct.lineF
  

  if keyword_set(graph_window) then wset=graph_window

;  white = FSC_Color('white')
;  black = FSC_Color('black')
;  gray = FSC_Color('gray')
;  blue  = FSC_Color('blue')
;  red   = FSC_Color('red')
;  green = FSC_Color('green')
;  magenta = FSC_Color('magenta')
;  orange = FSC_Color('orange')
;  cyan  = FSC_Color('cyan')
;  yellow= FSC_Color('gold')
;  purple=FSC_Color('purple')

  white = 16777215
  black = 0
  gray = 12500670
  blue  = 16711680
  red   = 255
  green = 65280
  magenta = 16711935
  orange = 42495
  cyan  = 16776960
  yellow= 55295
  purple= 15736992


  

  if lineF then begin
     
     if ((n_elements(xrange) eq 1) and (xrange[0] eq 0)) then begin
        index=where(abs(wave_grid-center) lt .01)
        xrange=[floor(min(wave_grid[index]*100))/100.,ceil(max(wave_grid[index]*100))/100. ]
     endif else begin
        index=where((wave_grid lt max(xrange)) and (wave_grid gt min(xrange)))
     endelse

     
     plot, wave_grid[index], sigSpecObs[index]*time/max(sigSpecObs[index]*time), background=white, $
           color=black, /nodata, xstyle=1, ystyle=8, $
           xrange=xrange,$
           xtitle='Wavelength [micron]', font=1, thick=2, charsize=2,position=[.15,.18,.8,.95], $
           ytitle='Transmission'
     

  endif else begin
     
     if ((n_elements(xrange) eq 1) and (xrange[0] eq 0)) then begin

        index=where(fltSpecObs gt 0.5)
        xrange=[floor(min(wave_grid[index]*100))/100.,ceil(max(wave_grid[index]*100))/100. ]
     endif else begin

        index=where((wave_grid lt max(xrange)) and (wave_grid gt min(xrange)))
     endelse

     plot, wave_grid[index], fltSpecObs[index], background=white, $
           color=black, /nodata, xstyle=1, ystyle=8, $
           xrange=xrange,$
           xtitle='Wavelength [micron]', font=1, thick=2, charsize=2,position=[.15,.18,.8,.95], $
           ytitle='Transmission'
     
     
  endelse

      ;*************************************
      ;*************************************
      ; plot the atmosphere and the throughput
      ;*************************************
      ;*************************************

  ;atmospheric trasparency
  oplot, wave_grid, tranSpecObs, color=purple, thick=2

  ;throughput
  oplot, wave_grid, tpSpecObs, color=green, thick=2


  xyouts, !X.CRange[1]-0.1*(!X.CRange[1]-!X.CRange[0]), 0.8, $
          'Atmosphere', color=purple, font=1, charsize=2, alignment=1
  xyouts, !X.CRange[1]-0.1*(!X.CRange[1]-!X.CRange[0]), 0.7, $
          'Throughput', color=green, font=1, charsize=2, alignment=1


      ;*************************************
      ;*************************************
      ; plot the sky and background spec
      ;*************************************
      ;*************************************


  ;instrumentally broadened night sky lines
  oplot, wave_grid[filt_index], $
         sqrt(bkSpecObs[filt_index]*time)/(2*max(sigSpecObs[filt_index]*time)), $
         color=red, thick=2

  ;signal
  oplot, wave_grid, sigSpecObs*time/(2*max(sigSpecObs*time)), color=blue, thick=2
  
  xyouts, !X.CRange[1]-0.1*(!X.CRange[1]-!X.CRange[0]), 0.6, $
          'Sky Resid', color=red, font=1, charsize=2, alignment=1
  xyouts, !X.CRange[1]-0.1*(!X.CRange[1]-!X.CRange[0]), 0.5, $
          'Science', color=blue, font=1, charsize=2, alignment=1
  
  axis, yaxis=1, ystyle=1, yrange=!Y.CRANGE*2*max(sigSpecObs*time[0]), ytitle='photons/pixel', $
        font=1,  charsize=2, color=black


end

pro plot_sn, struct, xrange, graph_window=graph_window

  if keyword_set(graph_window) then wset=graph_window

;  white = FSC_Color('white')
;  black = FSC_Color('black')
;  gray = FSC_Color('gray')
;  blue  = FSC_Color('blue')
;  red   = FSC_Color('red')
;  green = FSC_Color('green')
;  magenta = FSC_Color('magenta')
;  orange = FSC_Color('orange')
;  cyan  = FSC_Color('cyan')
;  yellow= FSC_Color('gold')
;  purple=FSC_Color('purple')

  white = 16777215
  black = 0
  gray = 12500670
  blue  = 16711680
  red   = 255
  green = 65280
  magenta = 16711935
  orange = 42495
  cyan  = 16776960
  yellow= 55295
  purple= 15736992

  

  ;make the structure into something useful
  wave_grid=*struct.wave_ptr
  sn_index=*struct.plot_index_ptr
  noiseSpecObs=*struct.noise_ptr
  snSpecObs=*struct.sn_ptr
  filt_index=*struct.filt_index_ptr
  signalSpecObs=*struct.signal_Ptr
  center=struct.center
  time=struct.time
  fltSpecObs=*struct.filt_ptr
  lineF=struct.lineF

  if lineF then begin

     if ((n_elements(xrange) eq 1) and (xrange[0] eq 0)) then begin

        index=where(abs(wave_grid-center) lt .01)
        xrange=[floor(min(wave_grid[index]*100))/100.,ceil(max(wave_grid[index]*100))/100. ]
     endif else begin

        index=where((wave_grid lt max(xrange)) and (wave_grid gt min(xrange)))
     endelse


     plot, wave_grid[index], fltSpecObs[index], background=white, $
           color=black, /nodata, xstyle=1, ystyle=4, $
           xrange=xrange,$
           xtitle='Wavelength [micron]', font=1, thick=2, charsize=2,position=[.15,.18,.8,.95]
         

  endif else begin

    if ((n_elements(xrange) eq 1) and (xrange[0] eq 0)) then begin

        index=where(fltSpecObs gt 0.5)
        xrange=[floor(min(wave_grid[index]*100))/100.,ceil(max(wave_grid[index]*100))/100. ]
     endif else begin

        index=where((wave_grid lt max(xrange)) and (wave_grid gt min(xrange)))
     endelse
     
     plot, wave_grid[index], fltSpecObs[index], background=white, $
           color=black, /nodata, xstyle=1, ystyle=4, $
           xrange=xrange,$
           xtitle='Wavelength [micron]', font=1, thick=2, charsize=2,position=[.15,.18,.8,.95]
                
  endelse

  oplot, wave_grid[filt_index], snSpecObs[filt_index]/(1.3*max(snSpecObs)), thick=2, color=green

  xyouts, !X.CRange[1]-0.1*(!X.CRange[1]-!X.CRange[0]), 0.8, $
          'S/N', color=green, font=1, charsize=2, alignment=1


  oplot, wave_grid[filt_index], signalSpecObs[filt_index]/(2*max(signalSpecObs)), thick=2, color=blue

  xyouts, !X.CRange[1]-0.1*(!X.CRange[1]-!X.CRange[0]), 0.7, $
          'Signal', color=blue, font=1, charsize=2, alignment=1

  oplot, wave_grid[filt_index], noiseSpecObs[filt_index]/(2*max(signalSpecObs)), thick=2, color=red

   xyouts, !X.CRange[1]-0.1*(!X.CRange[1]-!X.CRange[0]), 0.6, $
          'Noise', color=red, font=1, charsize=2, alignment=1


   axis, yaxis=0, ystyle=1, yrange=!Y.CRANGE*1.3*max(snSpecObs), ytitle='S/N per pixel', $
         font=1,  charsize=2, color=black

   axis, yaxis=1, ystyle=1, yrange=!Y.CRANGE*2*max(signalSpecObs), ytitle='photons/pixel', $
         font=1,  charsize=2, color=black

end
